@@ -3,17 +3,22 @@
 
 //! Cargo CLI support.
 
-use crate::output::OutputContext;
+use crate::{dispatch::CargoMessageFormatOpt, output::OutputContext};
 use camino::{Utf8Path, Utf8PathBuf};
 use clap::{ArgAction, Args};
 use std::{borrow::Cow, path::PathBuf};
 
-/// Options passed down to cargo.
-#[derive(Debug, Args)]
-#[command(
-    group = clap::ArgGroup::new("cargo-opts").multiple(true),
-)]
-pub(crate) struct CargoOptions {
+/// Build scope options: package selection, target selection, and feature
+/// selection.
+///
+/// These options determine which packages, targets, and features are built. In
+/// a rerun chain, these are inherited from the original run unless explicitly
+/// overridden.
+#[derive(Clone, Debug, Default, Args)]
+pub(crate) struct CargoBuildScopeOptions {
+    // ---
+    // Package selection
+    // ---
     /// Package to test
     #[arg(
         short = 'p',
@@ -35,6 +40,9 @@ pub(crate) struct CargoOptions {
     #[arg(long, group = "cargo-opts", help_heading = "Package selection")]
     all: bool,
 
+    // ---
+    // Target selection
+    // ---
     /// Test only this package's library unit tests
     #[arg(long, group = "cargo-opts", help_heading = "Target selection")]
     lib: bool,
@@ -75,6 +83,9 @@ pub(crate) struct CargoOptions {
     #[arg(long, group = "cargo-opts", help_heading = "Target selection")]
     all_targets: bool,
 
+    // ---
+    // Feature selection
+    // ---
     /// Space or comma separated list of features to activate
     #[arg(
         long,
@@ -91,6 +102,150 @@ pub(crate) struct CargoOptions {
     /// Do not activate the `default` feature
     #[arg(long, group = "cargo-opts", help_heading = "Feature selection")]
     no_default_features: bool,
+}
+
+impl CargoBuildScopeOptions {
+    /// Returns true if any build scope arguments were provided.
+    pub(crate) fn has_any(&self) -> bool {
+        let Self {
+            // Package selection.
+            packages,
+            workspace,
+            exclude,
+            all,
+            // Target selection.
+            lib,
+            bin,
+            bins,
+            example,
+            examples,
+            test,
+            tests,
+            bench,
+            benches,
+            all_targets,
+            // Feature selection.
+            features,
+            all_features,
+            no_default_features,
+        } = self;
+
+        !packages.is_empty()
+            || *workspace
+            || !exclude.is_empty()
+            || *all
+            || *lib
+            || !bin.is_empty()
+            || *bins
+            || !example.is_empty()
+            || *examples
+            || !test.is_empty()
+            || *tests
+            || !bench.is_empty()
+            || *benches
+            || *all_targets
+            || !features.is_empty()
+            || *all_features
+            || *no_default_features
+    }
+
+    /// Converts build scope args to a list of CLI arguments for storage.
+    pub(crate) fn to_cli_args(&self) -> Vec<&str> {
+        let Self {
+            // Package selection.
+            packages,
+            workspace,
+            exclude,
+            all,
+            // Target selection.
+            lib,
+            bin,
+            bins,
+            example,
+            examples,
+            test,
+            tests,
+            bench,
+            benches,
+            all_targets,
+            // Feature selection.
+            features,
+            all_features,
+            no_default_features,
+        } = self;
+
+        let mut args = Vec::new();
+
+        // Package selection.
+        for pkg in packages {
+            args.extend(["--package", pkg.as_str()]);
+        }
+        if *workspace {
+            args.push("--workspace");
+        }
+        for exc in exclude {
+            args.extend(["--exclude", exc.as_str()]);
+        }
+        if *all {
+            args.push("--all");
+        }
+
+        // Target selection.
+        if *lib {
+            args.push("--lib");
+        }
+        for b in bin {
+            args.extend(["--bin", b.as_str()]);
+        }
+        if *bins {
+            args.push("--bins");
+        }
+        for ex in example {
+            args.extend(["--example", ex.as_str()]);
+        }
+        if *examples {
+            args.push("--examples");
+        }
+        for t in test {
+            args.extend(["--test", t.as_str()]);
+        }
+        if *tests {
+            args.push("--tests");
+        }
+        for b in bench {
+            args.extend(["--bench", b.as_str()]);
+        }
+        if *benches {
+            args.push("--benches");
+        }
+        if *all_targets {
+            args.push("--all-targets");
+        }
+
+        // Feature selection.
+        for f in features {
+            args.extend(["--features", f.as_str()]);
+        }
+        if *all_features {
+            args.push("--all-features");
+        }
+        if *no_default_features {
+            args.push("--no-default-features");
+        }
+
+        args
+    }
+}
+
+/// Options passed down to cargo.
+#[derive(Debug, Default, Args)]
+#[command(
+    group = clap::ArgGroup::new("cargo-opts").multiple(true),
+)]
+pub(crate) struct CargoOptions {
+    /// Build scope options (package, target, and feature selection).
+    #[command(flatten)]
+    pub(crate) build_scope: CargoBuildScopeOptions,
 
     // jobs is handled by test runner
     /// Number of build jobs to run
@@ -167,6 +322,20 @@ pub(crate) struct CargoOptions {
     #[arg(long, group = "cargo-opts", help_heading = "Manifest options")]
     offline: bool,
 
+    /// Cargo message format.
+    ///
+    /// Controls how Cargo reports build messages and whether to forward JSON to
+    /// stdout. Multiple JSON options can be comma-separated or repeated.
+    #[arg(
+        long,
+        value_enum,
+        value_name = "FMT",
+        value_delimiter = ',',
+        group = "cargo-opts",
+        help_heading = "Other Cargo options"
+    )]
+    pub(crate) cargo_message_format: Vec<CargoMessageFormatOpt>,
+
     //  TODO: doc?
     // no-run is handled by test runner
     /// Do not print cargo log messages (specify twice for no Cargo output at all)
@@ -238,71 +407,30 @@ impl<'a> CargoCli<'a> {
         self
     }
 
+    /// Adds build scope options (package, target, and feature selection) to the
+    /// command.
+    pub(crate) fn add_build_scope_options(
+        &mut self,
+        scope: &'a CargoBuildScopeOptions,
+    ) -> &mut Self {
+        for arg in scope.to_cli_args() {
+            self.add_arg(arg);
+        }
+        self
+    }
+
     pub(crate) fn add_options(&mut self, options: &'a CargoOptions) -> &mut Self {
-        // ---
-        // Package selection
-        // ---
-        self.add_args(
-            options
-                .packages
-                .iter()
-                .flat_map(|s| ["--package", s.as_str()]),
-        );
-        if options.workspace {
-            self.add_arg("--workspace");
-        }
-        self.add_args(
-            options
-                .exclude
-                .iter()
-                .flat_map(|s| ["--exclude", s.as_str()]),
-        );
-        if options.all {
-            self.add_arg("--all");
-        }
-
-        // ---
-        // Target selection
-        // ---
-        if options.lib {
-            self.add_arg("--lib");
-        }
-        self.add_args(options.bin.iter().flat_map(|s| ["--bin", s.as_str()]));
-        if options.bins {
-            self.add_arg("--bins");
-        }
-        self.add_args(
-            options
-                .example
-                .iter()
-                .flat_map(|s| ["--example", s.as_str()]),
-        );
-        if options.examples {
-            self.add_arg("--examples");
-        }
-        self.add_args(options.test.iter().flat_map(|s| ["--test", s.as_str()]));
-        if options.tests {
-            self.add_arg("--tests");
-        }
-        self.add_args(options.bench.iter().flat_map(|s| ["--bench", s.as_str()]));
-        if options.benches {
-            self.add_arg("--benches");
-        }
-        if options.all_targets {
-            self.add_arg("--all-targets");
-        }
-
-        // ---
-        // Feature selection
-        // ---
-        self.add_args(options.features.iter().flat_map(|s| ["--features", s]));
-        if options.all_features {
-            self.add_arg("--all-features");
-        }
-        if options.no_default_features {
-            self.add_arg("--no-default-features");
-        }
+        self.add_build_scope_options(&options.build_scope);
+        self.add_non_build_scope_options(options);
+        self
+    }
 
+    /// Adds non-build-scope options from `CargoOptions`.
+    ///
+    /// This includes compilation options, manifest options, and other cargo
+    /// options. Used when inheriting build scope from a rerun but using the
+    /// current run's other options.
+    pub(crate) fn add_non_build_scope_options(&mut self, options: &'a CargoOptions) -> &mut Self {
         // ---
         // Compilation options
         // ---
@@ -394,13 +522,13 @@ impl<'a> CargoCli<'a> {
         self
     }
 
-    fn add_owned_arg(&mut self, arg: String) {
+    pub(crate) fn add_owned_arg(&mut self, arg: String) {
         self.args.push(Cow::Owned(arg));
     }
 
     pub(crate) fn all_args(&self) -> Vec<&str> {
         let mut all_args = vec![self.cargo_path.as_str(), self.command];
-        all_args.extend(self.args.iter().map(|s| s.as_ref()));
+        all_args.extend(self.args.iter().map(|s| &**s));
         all_args
     }
 
