@@ -17,6 +17,7 @@ use crate::{
     output::OutputWriter,
     reuse_build::ReuseBuildOpts,
 };
+use camino::Utf8Path;
 use clap::{Args, builder::BoolishValueParser};
 use nextest_filtering::{FiltersetKind, ParseContext};
 use nextest_runner::{
@@ -29,10 +30,9 @@ use nextest_runner::{
     input::InputHandlerKind,
     list::{BinaryList, TestExecuteContext, TestList},
     record::{
-        ComputedRerunInfo, RecordOpts, RecordReader, RecordRetentionPolicy, RecordSession,
-        RecordSessionConfig, RunIdSelector, RunStore, Styles as RecordStyles,
-        format::{RECORD_FORMAT_VERSION, RerunRootInfo},
-        records_cache_dir,
+        ComputedRerunInfo, PortableRecording, RecordOpts, RecordReader, RecordRetentionPolicy,
+        RecordSession, RecordSessionConfig, RerunRootInfo, RunIdOrRecordingSelector, RunIdSelector,
+        RunStore, STORE_FORMAT_VERSION, Styles as RecordStyles, records_state_dir,
     },
     reporter::{
         FinalStatusLevel, MaxProgressRunning, ReporterBuilder, ShowTerminalProgress, StatusLevel,
@@ -62,18 +62,19 @@ pub(crate) struct RunOpts {
 
     /// Rerun tests that failed or didn't complete in a previous recorded run.
     ///
-    /// Accepts a run ID (full UUID, short prefix like "abc1234", or "latest").
+    /// Accepts a run ID (full UUID, short prefix like "abc1234", or "latest"),
+    /// or a path to a portable recording file (ending in `.zip`).
     /// Only tests that didn't pass in the specified run will be executed.
     ///
     /// New tests (not in the parent run) are also included by default.
     #[arg(
         long,
         short = 'R',
-        value_name = "RUN_ID",
+        value_name = "RUN_ID_OR_RECORDING",
         alias = "run-id",
         help_heading = "Filter options"
     )]
-    pub(crate) rerun: Option<RunIdSelector>,
+    pub(crate) rerun: Option<RunIdOrRecordingSelector>,
 
     #[clap(flatten)]
     pub(crate) build_filter: TestBuildFilter,
@@ -463,16 +464,34 @@ fn no_run_no_capture_reasons(no_run: bool, no_capture: bool) -> Option<&'static
 #[derive(Debug, Default, Args)]
 pub(crate) struct ReporterCommonOpts {
     /// Output stdout and stderr on failure.
-    #[arg(long, value_enum, value_name = "WHEN", env = "NEXTEST_FAILURE_OUTPUT")]
+    #[arg(
+        long,
+        value_enum,
+        value_name = "WHEN",
+        env = "NEXTEST_FAILURE_OUTPUT",
+        help_heading = "Reporter options"
+    )]
     pub(crate) failure_output: Option<TestOutputDisplayOpt>,
 
     /// Output stdout and stderr on success.
-    #[arg(long, value_enum, value_name = "WHEN", env = "NEXTEST_SUCCESS_OUTPUT")]
+    #[arg(
+        long,
+        value_enum,
+        value_name = "WHEN",
+        env = "NEXTEST_SUCCESS_OUTPUT",
+        help_heading = "Reporter options"
+    )]
     pub(crate) success_output: Option<TestOutputDisplayOpt>,
 
     // status_level does not conflict with --no-capture because pass vs skip still makes sense.
     /// Test statuses to output.
-    #[arg(long, value_enum, value_name = "LEVEL", env = "NEXTEST_STATUS_LEVEL")]
+    #[arg(
+        long,
+        value_enum,
+        value_name = "LEVEL",
+        env = "NEXTEST_STATUS_LEVEL",
+        help_heading = "Reporter options"
+    )]
     pub(crate) status_level: Option<StatusLevelOpt>,
 
     /// Test statuses to output at the end of the run.
@@ -480,7 +499,8 @@ pub(crate) struct ReporterCommonOpts {
         long,
         value_enum,
         value_name = "LEVEL",
-        env = "NEXTEST_FINAL_STATUS_LEVEL"
+        env = "NEXTEST_FINAL_STATUS_LEVEL",
+        help_heading = "Reporter options"
     )]
     pub(crate) final_status_level: Option<FinalStatusLevelOpt>,
 
@@ -492,7 +512,12 @@ pub(crate) struct ReporterCommonOpts {
     ///
     /// This option has no effect with **--no-capture**, since that passes
     /// through standard output and standard error.
-    #[arg(long, env = "NEXTEST_NO_OUTPUT_INDENT", value_parser = BoolishValueParser::new())]
+    #[arg(
+        long,
+        env = "NEXTEST_NO_OUTPUT_INDENT",
+        value_parser = BoolishValueParser::new(),
+        help_heading = "Reporter options"
+    )]
     pub(crate) no_output_indent: bool,
 }
 
@@ -834,7 +859,7 @@ impl App {
     pub(crate) fn exec_run(
         &self,
         no_capture: bool,
-        rerun: Option<&RunIdSelector>,
+        rerun: Option<&RunIdOrRecordingSelector>,
         runner_opts: &TestRunnerOpts,
         reporter_opts: &ReporterOpts,
         cli_args: Vec<String>,
@@ -921,13 +946,20 @@ impl App {
         let mut test_filter_builder = self
             .build_filter
             .make_test_filter_builder(NextestRunMode::Test, filter_exprs)?;
-        let (rerun_state, expected_outstanding) = if let Some(run_id_selector) = rerun {
-            let (rerun_state, outstanding_tests) = self.resolve_rerun(run_id_selector)?;
-            let expected = outstanding_tests.expected_test_ids();
-            test_filter_builder.set_outstanding_tests(outstanding_tests);
-            (Some(rerun_state), Some(expected))
-        } else {
-            (None, None)
+        let (rerun_state, expected_outstanding) = match rerun {
+            Some(RunIdOrRecordingSelector::RunId(selector)) => {
+                let (rerun_state, outstanding_tests) = self.resolve_rerun(selector)?;
+                let expected = outstanding_tests.expected_test_ids();
+                test_filter_builder.set_outstanding_tests(outstanding_tests);
+                (Some(rerun_state), Some(expected))
+            }
+            Some(RunIdOrRecordingSelector::RecordingPath(path)) => {
+                let (rerun_state, outstanding_tests) = self.resolve_rerun_from_archive(path)?;
+                let expected = outstanding_tests.expected_test_ids();
+                test_filter_builder.set_outstanding_tests(outstanding_tests);
+                (Some(rerun_state), Some(expected))
+            }
+            None => (None, None),
         };
 
         // Start running Cargo commands at this point, once all initial
@@ -1392,11 +1424,11 @@ impl App {
         &self,
         run_id_selector: &RunIdSelector,
     ) -> Result<(RerunState, ComputedRerunInfo), ExpectedError> {
-        let cache_dir = records_cache_dir(&self.base.workspace_root)
-            .map_err(|err| ExpectedError::RecordCacheDirNotFound { err })?;
+        let state_dir = records_state_dir(&self.base.workspace_root)
+            .map_err(|err| ExpectedError::RecordStateDirNotFound { err })?;
 
         let store =
-            RunStore::new(&cache_dir).map_err(|err| ExpectedError::RecordSetupError { err })?;
+            RunStore::new(&state_dir).map_err(|err| ExpectedError::RecordSetupError { err })?;
 
         // First, acquire a shared lock to resolve the run ID and read the
         // parent run.
@@ -1420,11 +1452,13 @@ impl App {
             .find(|r| r.run_id == parent_run_id)
             .expect("resolved run ID must be in the snapshot");
 
-        if run_info.store_format_version != RECORD_FORMAT_VERSION {
-            return Err(ExpectedError::UnsupportedStoreFormatVersion {
+        if let Err(incompatibility) = run_info
+            .store_format_version
+            .check_readable_by(STORE_FORMAT_VERSION)
+        {
+            return Err(ExpectedError::StoreVersionIncompatible {
                 run_id: parent_run_id,
-                found: run_info.store_format_version,
-                supported: RECORD_FORMAT_VERSION,
+                incompatibility,
             });
         }
 
@@ -1446,6 +1480,34 @@ impl App {
             outstanding_tests,
         ))
     }
+
+    fn resolve_rerun_from_archive(
+        &self,
+        archive_path: &Utf8Path,
+    ) -> Result<(RerunState, ComputedRerunInfo), ExpectedError> {
+        let mut archive = PortableRecording::open(archive_path)
+            .map_err(|err| ExpectedError::PortableRecordingReadError { err })?;
+
+        let run_info = archive.run_info();
+        let parent_run_id = run_info.run_id;
+
+        let (outstanding_tests, root_info) = ComputedRerunInfo::compute_from_archive(&mut archive)
+            .map_err(|err| ExpectedError::RecordReadError { err })?;
+
+        // If the archive is itself a rerun, use its root_info.
+        // Otherwise, this archive is the root of the chain.
+        let root_info = root_info.unwrap_or_else(|| {
+            RerunRootInfo::new(parent_run_id, run_info.build_scope_args.clone())
+        });
+
+        Ok((
+            RerunState {
+                parent_run_id,
+                root_info,
+            },
+            outstanding_tests,
+        ))
+    }
 }
 
 #[derive(Debug)]
