@@ -165,6 +165,32 @@ fn test_argument_parsing() {
         "cargo nextest replay --no-capture --no-output-indent",
         "cargo nextest replay --status-level pass",
         "cargo nextest replay --final-status-level flaky",
+        // ---
+        // Store commands
+        // ---
+        "cargo nextest store list",
+        "cargo nextest store info latest",
+        "cargo nextest store info abc123",
+        "cargo nextest store info -R latest",
+        "cargo nextest store info --run-id abc123",
+        "cargo nextest store info archive.zip",
+        "cargo nextest store info /path/to/run.zip",
+        "cargo nextest store info -R ./relative/path.zip",
+        "cargo nextest store export latest",
+        "cargo nextest store export abc123",
+        "cargo nextest store export -R latest",
+        "cargo nextest store export --run-id abc123",
+        "cargo nextest store export latest --archive-file output.zip",
+        "cargo nextest store export --run-id abc123 --archive-file /path/to/archive.zip",
+        "cargo nextest store prune",
+        "cargo nextest store prune --dry-run",
+        // ---
+        // Rerun with archive paths
+        // ---
+        "cargo nextest run --rerun archive.zip",
+        "cargo nextest run --rerun /path/to/run.zip",
+        "cargo nextest run -R ./relative/path.zip",
+        "cargo nextest run --rerun ../parent/run.zip",
     ];
 
     let invalid: &[(&'static str, ErrorKind)] = &[
@@ -326,6 +352,47 @@ fn test_argument_parsing() {
             "cargo nextest self update --rc --version 0.9.100",
             ArgumentConflict,
         ),
+        // ---
+        // Store command conflicts
+        // ---
+        ("cargo nextest store info", MissingRequiredArgument),
+        (
+            "cargo nextest store info latest -R abc123",
+            ArgumentConflict,
+        ),
+        (
+            "cargo nextest store info latest --run-id abc123",
+            ArgumentConflict,
+        ),
+        ("cargo nextest store export", MissingRequiredArgument),
+        (
+            "cargo nextest store export latest -R abc123",
+            ArgumentConflict,
+        ),
+        (
+            "cargo nextest store export latest --run-id abc123",
+            ArgumentConflict,
+        ),
+        // store export does not accept archive paths (uses RunIdSelector, not
+        // RunIdOrRecordingSelector).
+        ("cargo nextest store export archive.zip", ValueValidation),
+        (
+            "cargo nextest store export /path/to/run.zip",
+            ValueValidation,
+        ),
+        (
+            "cargo nextest store export -R ./relative.zip",
+            ValueValidation,
+        ),
+        // store export --archive-file must have .zip extension.
+        (
+            "cargo nextest store export latest --archive-file output.txt",
+            ValueValidation,
+        ),
+        (
+            "cargo nextest store export latest --archive-file /path/to/archive.tar.zst",
+            ValueValidation,
+        ),
     ];
 
     // Unset all NEXTEST_ env vars because they can conflict with the try_parse_from below.
