@@ -8,23 +8,58 @@ use nextest_filtering::errors::FiltersetParseErrors;
 use nextest_metadata::NextestExitCode;
 use nextest_runner::{
     config::core::{ConfigExperimental, ToolName},
-    errors::{UserConfigError, *},
+    errors::{
+        PortableRecordingError, PortableRecordingReadError, RecordReadError, RecordSetupError,
+        RunIdResolutionError, RunStoreError, StateDirError, TestListFromSummaryError,
+        UserConfigError, *,
+    },
     helpers::{format_interceptor_too_many_tests, plural},
     indenter::DisplayIndented,
     list::OwnedTestInstanceId,
+    record::{ReplayabilityStatus, RunListAlignment, StoreVersionIncompatibility},
     redact::Redactor,
     run_mode::NextestRunMode,
     runner::{DebuggerCommand, TracerCommand},
 };
 use owo_colors::{OwoColorize, Style};
+use quick_junit::ReportUuid;
 use semver::Version;
 use std::{error::Error, io, process::ExitStatus, string::FromUtf8Error};
-use swrite::{SWrite, swriteln};
+use swrite::{SWrite, swrite, swriteln};
 use thiserror::Error;
-use tracing::{Level, error, info};
+use tracing::{Level, error, info, warn};
 
 pub(crate) type Result<T, E = ExpectedError> = std::result::Result<T, E>;
 
+/// Error when combining incompatible cargo message format options.
+#[derive(Clone, Debug, PartialEq, Eq, Error)]
+pub enum CargoMessageFormatError {
+    /// Conflicting base formats specified.
+    #[error(
+        "conflicting message formats: `{first}` and `{second}` cannot be combined\n\
+         (only JSON modifiers like `json-diagnostic-short` can be combined)"
+    )]
+    ConflictingBaseFormats {
+        first: &'static str,
+        second: &'static str,
+    },
+
+    /// JSON modifier used with non-JSON base format.
+    #[error(
+        "cannot combine {modifiers} with `{base}` format\n\
+         (JSON modifiers can only be used with JSON output)"
+    )]
+    JsonModifierWithNonJson {
+        /// Comma-separated list of modifiers, each wrapped in backticks.
+        modifiers: &'static str,
+        base: &'static str,
+    },
+
+    /// Duplicate option specified.
+    #[error("duplicate message format option: `{option}`")]
+    Duplicate { option: &'static str },
+}
+
 #[derive(Debug)]
 #[doc(hidden)]
 pub enum ReuseBuildKind {
@@ -220,7 +255,15 @@ pub enum ExpectedError {
     #[error("setup script failed")]
     SetupScriptFailed,
     #[error("test run failed")]
-    TestRunFailed,
+    TestRunFailed {
+        /// Whether the user can rerun failing tests with `-R latest`.
+        rerun_available: bool,
+    },
+    #[error("rerun tests outstanding")]
+    RerunTestsOutstanding {
+        /// The number of tests that were not seen during this rerun.
+        count: usize,
+    },
     #[error("no tests to run")]
     NoTestsRun {
         /// The run mode (test or benchmark).
@@ -312,6 +355,21 @@ pub enum ExpectedError {
         config_file: Utf8PathBuf,
         missing: Vec<ConfigExperimental>,
     },
+    #[error("could not determine state directory for recording")]
+    RecordStateDirNotFound {
+        #[source]
+        err: StateDirError,
+    },
+    #[error("error setting up recording")]
+    RecordSetupError {
+        #[source]
+        err: RunStoreError,
+    },
+    #[error("error setting up recording session")]
+    RecordSessionSetupError {
+        #[source]
+        err: RecordSetupError,
+    },
     #[error("filterset parse error")]
     FiltersetParseError {
         all_errors: Vec<FiltersetParseErrors>,
@@ -341,6 +399,11 @@ pub enum ExpectedError {
         #[from]
         err: FormatVersionError,
     },
+    #[error("invalid cargo message format")]
+    CargoMessageFormatError {
+        #[from]
+        err: CargoMessageFormatError,
+    },
     #[error("extract read error")]
     DebugExtractReadError {
         kind: &'static str,
@@ -354,6 +417,43 @@ pub enum ExpectedError {
         #[source]
         err: std::io::Error,
     },
+    #[error("error extracting from portable recording: {message}")]
+    DebugExtractRecordingError { message: String },
+    #[error("run ID resolution error")]
+    RunIdResolutionError {
+        #[source]
+        err: RunIdResolutionError,
+    },
+    #[error("error reading recorded run")]
+    RecordReadError {
+        #[source]
+        err: RecordReadError,
+    },
+    #[error("run {run_id} has incompatible store format: {incompatibility}")]
+    StoreVersionIncompatible {
+        run_id: ReportUuid,
+        incompatibility: StoreVersionIncompatibility,
+    },
+    #[error("error creating portable recording")]
+    PortableRecordingError {
+        #[source]
+        err: PortableRecordingError,
+    },
+    #[error("error reading portable recording")]
+    PortableRecordingReadError {
+        #[source]
+        err: PortableRecordingReadError,
+    },
+    #[error("error reconstructing test list from archive")]
+    TestListFromSummaryError {
+        #[source]
+        err: TestListFromSummaryError,
+    },
+    #[error("write error")]
+    WriteError {
+        #[source]
+        err: std::io::Error,
+    },
 }
 
 impl ExpectedError {
@@ -458,8 +558,8 @@ impl ExpectedError {
         Self::SetupScriptFailed
     }
 
-    pub(crate) fn test_run_failed() -> Self {
-        Self::TestRunFailed
+    pub(crate) fn test_run_failed(rerun_available: bool) -> Self {
+        Self::TestRunFailed { rerun_available }
     }
 
     pub(crate) fn test_binary_args_parse_error(reason: &'static str, args: Vec<String>) -> Self {
@@ -497,7 +597,9 @@ impl ExpectedError {
             | Self::SignalHandlerSetupError { .. }
             | Self::ShowTestGroupsError { .. }
             | Self::InvalidMessageFormatVersion { .. }
-            | Self::DebugExtractReadError { .. } => NextestExitCode::SETUP_ERROR,
+            | Self::DebugExtractReadError { .. }
+            | Self::DebugExtractRecordingError { .. }
+            | Self::CargoMessageFormatError { .. } => NextestExitCode::SETUP_ERROR,
             Self::ConfigParseError { err } => {
                 // Experimental features not being enabled are their own error.
                 match err.kind() {
@@ -521,7 +623,8 @@ impl ExpectedError {
                 NextestExitCode::BUILD_FAILED
             }
             Self::SetupScriptFailed => NextestExitCode::SETUP_SCRIPT_FAILED,
-            Self::TestRunFailed => NextestExitCode::TEST_RUN_FAILED,
+            Self::TestRunFailed { .. } => NextestExitCode::TEST_RUN_FAILED,
+            Self::RerunTestsOutstanding { .. } => NextestExitCode::RERUN_TESTS_OUTSTANDING,
             Self::NoTestsRun { .. } => NextestExitCode::NO_TESTS_RUN,
             Self::DebuggerNoTests { .. } | Self::DebuggerTooManyTests { .. }
             | Self::TracerNoTests { .. } | Self::TracerTooManyTests { .. } => NextestExitCode::SETUP_ERROR,
@@ -538,6 +641,16 @@ impl ExpectedError {
             | Self::ConfigExperimentalFeaturesNotEnabled { .. } => {
                 NextestExitCode::EXPERIMENTAL_FEATURE_NOT_ENABLED
             }
+            Self::RecordStateDirNotFound { .. }
+            | Self::RecordSetupError { .. }
+            | Self::RecordSessionSetupError { .. }
+            | Self::RunIdResolutionError { .. }
+            | Self::RecordReadError { .. }
+            | Self::StoreVersionIncompatible { .. }
+            | Self::TestListFromSummaryError { .. } => NextestExitCode::SETUP_ERROR,
+            Self::PortableRecordingError { .. } => NextestExitCode::ARCHIVE_CREATION_FAILED,
+            Self::PortableRecordingReadError { .. } => NextestExitCode::SETUP_ERROR,
+            Self::WriteError { .. } => NextestExitCode::WRITE_OUTPUT_ERROR,
             Self::FiltersetParseError { .. } => NextestExitCode::INVALID_FILTERSET,
         }
     }
@@ -723,25 +836,27 @@ impl ExpectedError {
                                 "- unknown scripts specified (known scripts: {known_scripts_str}):",
                             );
                             for error in unknown_scripts {
-                                errors_str.push_str(&format!(
+                                swrite!(
+                                    errors_str,
                                     "  - script `{}` specified within profile `{}`\n",
                                     error.name.style(styles.bold),
                                     error.profile_name.style(styles.bold)
-                                ));
+                                );
                             }
                         }
 
                         if !wrong_script_types.is_empty() {
                             swriteln!(errors_str, "- scripts specified with incorrect type:");
                             for error in wrong_script_types {
-                                errors_str.push_str(&format!(
+                                swrite!(
+                                    errors_str,
                                     "  - script `{}` specified within profile `{}` as {}, \
                                      but is actually {}\n",
                                     error.name.style(styles.bold),
                                     error.profile_name.style(styles.bold),
                                     error.attempted.style(styles.bold),
                                     error.actual.style(styles.bold)
-                                ));
+                                );
                             }
                         }
 
@@ -782,7 +897,7 @@ impl ExpectedError {
                             .join(", ");
 
                         error!(
-                            "for config file `{}`{}, unknown experimental features defined:
+                            "for config file `{}`{}, unknown experimental features defined: \
                              {unknown_str} (known features: {known_str}):",
                             err.config_file(),
                             provided_by_tool(err.tool()),
@@ -934,8 +1049,33 @@ impl ExpectedError {
                 error!("setup script failed");
                 None
             }
-            Self::TestRunFailed => {
+            Self::TestRunFailed { rerun_available } => {
                 error!("test run failed");
+                if *rerun_available {
+                    info!(
+                        target: "cargo_nextest::no_heading",
+                        "(hint: {} to rerun failing tests)",
+                        "cargo nextest run -R latest".style(styles.bold),
+                    );
+                }
+                None
+            }
+            Self::RerunTestsOutstanding { count } => {
+                warn!(
+                    "{} outstanding {} still {}",
+                    count.style(styles.bold),
+                    // We only support reruns for test mode.
+                    plural::tests_str(NextestRunMode::Test, *count),
+                    plural::remain_str(*count),
+                );
+                // Advice to run `cargo nextest run -R latest` might not always
+                // be complete due to the expanded build scope or disappearing
+                // tests. We just hint that the user can continue doing reruns.
+                info!(
+                    target: "cargo_nextest::no_heading",
+                    "(hint: {} to continue rerunning)",
+                    "cargo nextest run -R latest".style(styles.bold),
+                );
                 None
             }
             Self::NoTestsRun { mode, is_default } => {
@@ -1068,6 +1208,18 @@ impl ExpectedError {
                 );
                 None
             }
+            Self::RecordStateDirNotFound { err } => {
+                error!("could not determine state directory for recording test runs");
+                Some(err as &dyn Error)
+            }
+            Self::RecordSetupError { err } => {
+                error!("error setting up recording");
+                Some(err as &dyn Error)
+            }
+            Self::RecordSessionSetupError { err } => {
+                error!("error setting up recording session");
+                Some(err as &dyn Error)
+            }
             Self::FiltersetParseError { all_errors } => {
                 for errors in all_errors {
                     for single_error in &errors.errors {
@@ -1111,6 +1263,10 @@ impl ExpectedError {
                 error!("error parsing message format version");
                 Some(err as &dyn Error)
             }
+            Self::CargoMessageFormatError { err } => {
+                error!("invalid --cargo-message-format: {err}");
+                None
+            }
             Self::DebugExtractReadError { kind, path, err } => {
                 error!("error reading {kind} file `{}`", path.style(styles.bold),);
                 Some(err as &dyn Error)
@@ -1119,6 +1275,96 @@ impl ExpectedError {
                 error!("error writing {format} output");
                 Some(err as &dyn Error)
             }
+            Self::DebugExtractRecordingError { message } => {
+                error!("{message}");
+                None
+            }
+            Self::RunIdResolutionError { err } => {
+                match &err {
+                    RunIdResolutionError::NotFound { prefix } => {
+                        error!(
+                            "no recorded run found matching `{}`",
+                            prefix.style(styles.bold)
+                        );
+                    }
+                    RunIdResolutionError::Ambiguous {
+                        prefix,
+                        count,
+                        candidates,
+                        run_id_index,
+                    } => {
+                        error!(
+                            "prefix `{}` is ambiguous, matches {} {}:",
+                            prefix.style(styles.bold),
+                            count.style(styles.bold),
+                            plural::runs_str(*count)
+                        );
+                        let redactor = if crate::output::should_redact() {
+                            Redactor::for_snapshot_testing()
+                        } else {
+                            Redactor::noop()
+                        };
+                        let alignment = RunListAlignment::from_runs(candidates);
+                        for candidate in candidates {
+                            info!(
+                                target: "cargo_nextest::no_heading",
+                                "{}",
+                                candidate.display(run_id_index, &ReplayabilityStatus::Replayable, alignment, &styles.record_styles, &redactor)
+                            );
+                        }
+                        if *count > candidates.len() {
+                            let remaining = *count - candidates.len();
+                            info!(
+                                target: "cargo_nextest::no_heading",
+                                "  ... and {} more {}",
+                                remaining.style(styles.bold),
+                                plural::runs_str(remaining)
+                            );
+                        }
+                    }
+                    RunIdResolutionError::InvalidPrefix { prefix } => {
+                        error!(
+                            "prefix `{}` contains invalid characters (expected hexadecimal)",
+                            prefix.style(styles.bold)
+                        );
+                    }
+                    RunIdResolutionError::NoRuns => {
+                        error!("no recorded runs exist");
+                    }
+                }
+                None
+            }
+            Self::RecordReadError { err } => {
+                error!("error reading recorded run");
+                Some(err as &dyn Error)
+            }
+            Self::PortableRecordingError { err } => {
+                error!("error creating portable recording");
+                Some(err as &dyn Error)
+            }
+            Self::PortableRecordingReadError { err } => {
+                error!("error reading portable recording");
+                Some(err as &dyn Error)
+            }
+            Self::StoreVersionIncompatible {
+                run_id,
+                incompatibility,
+            } => {
+                error!(
+                    "run {} has incompatible store format: {}",
+                    run_id.style(styles.bold),
+                    incompatibility,
+                );
+                None
+            }
+            Self::TestListFromSummaryError { err } => {
+                error!("error reconstructing test list from archived summary");
+                Some(err as &dyn Error)
+            }
+            Self::WriteError { err } => {
+                error!("write error");
+                Some(err as &dyn Error)
+            }
         };
 
         while let Some(err) = next_error {
