@@ -9,10 +9,10 @@ use crate::{
     errors::{
         DisplayErrorChain, HostPlatformDetectError, RustBuildMetaParseError, TargetTripleError,
     },
+    indenter::DisplayIndented,
     reuse_build::{LibdirMapper, PlatformLibdirMapper},
 };
 use camino::{Utf8Path, Utf8PathBuf};
-use indent_write::indentable::Indented;
 use nextest_metadata::{
     BuildPlatformsSummary, HostPlatformSummary, PlatformLibdirSummary, PlatformLibdirUnavailable,
     TargetPlatformSummary,
@@ -317,11 +317,11 @@ fn detect_host_platform() -> Result<Platform, HostPlatformDetectError> {
                              - `rustc -vV` stderr:\n{}",
                             output.status,
                             build_target.triple().as_str(),
-                            Indented {
+                            DisplayIndented {
                                 item: String::from_utf8_lossy(&output.stdout),
                                 indent: "  "
                             },
-                            Indented {
+                            DisplayIndented {
                                 item: String::from_utf8_lossy(&output.stderr),
                                 indent: "  "
                             },
@@ -497,6 +497,21 @@ impl PlatformLibdir {
     }
 }
 
+/// Detects the host platform for use in tests.
+///
+/// Prefer this over `Platform::build_target()` in tests to ensure we're testing
+/// with the same platform detection logic used in production.
+#[cfg(test)]
+pub(crate) fn detect_host_platform_for_tests() -> Platform {
+    use crate::RustcCli;
+
+    HostPlatform::detect(PlatformLibdir::from_rustc_stdout(
+        RustcCli::print_host_libdir().read(),
+    ))
+    .expect("host platform detection should succeed in tests")
+    .platform
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
