@@ -10,7 +10,8 @@ use crate::{
     errors::RecordReadError,
     list::OwnedTestInstanceId,
     record::{
-        CoreEventKind, OutputEventKind, RecordReader, TestEventKindSummary,
+        CoreEventKind, OutputEventKind, PortableRecording, RecordReader, StoreReader,
+        TestEventKindSummary,
         format::{RerunInfo, RerunRootInfo, RerunTestSuiteInfo},
     },
 };
@@ -20,7 +21,10 @@ use nextest_metadata::{
     TestListSummary,
 };
 use quick_junit::ReportUuid;
-use std::collections::{BTreeSet, HashMap};
+use std::{
+    borrow::Borrow,
+    collections::{BTreeSet, HashMap},
+};
 
 /// Trait abstracting over test list access for rerun computation.
 ///
@@ -263,6 +267,39 @@ impl ComputedRerunInfo {
         ))
     }
 
+    /// Computes outstanding tests from a portable recording.
+    ///
+    /// If the archive is itself a rerun, also returns information about the
+    /// root of the chain.
+    pub fn compute_from_archive(
+        archive: &mut PortableRecording,
+    ) -> Result<(Self, Option<RerunRootInfo>), RecordReadError> {
+        let mut store = archive
+            .open_store()
+            .map_err(RecordReadError::PortableRecording)?;
+        let rerun_info = store.read_rerun_info()?;
+        let test_list = store.read_test_list()?;
+        // No need to load dictionaries since we're not using them.
+
+        // Read events from the archive's run log.
+        let run_log = archive
+            .read_run_log()
+            .map_err(RecordReadError::PortableRecording)?;
+        let outcomes = collect_from_events(run_log.events()?.map(|r| r.map(|e| e.kind)))?;
+
+        let prev_test_suites = rerun_info.as_ref().map(|info| &info.test_suites);
+        let new_test_suites = compute_outstanding_pure(prev_test_suites, &test_list, &outcomes);
+
+        let root_info = rerun_info.map(|info| info.root_info);
+
+        Ok((
+            Self {
+                test_suites: new_test_suites,
+            },
+            root_info,
+        ))
+    }
+
     /// Consumes self, converting to a [`RerunInfo`] for storage.
     pub fn into_rerun_info(self, parent_run_id: ReportUuid, root_info: RerunRootInfo) -> RerunInfo {
         RerunInfo {
@@ -372,28 +409,28 @@ impl TestEventOutcomes {
     /// (had any event: started, finished, or skipped).
     fn collect(reader: &mut RecordReader) -> Result<Self, RecordReadError> {
         reader.load_dictionaries()?;
-
-        let events: Vec<_> = reader.events()?.collect::<Result<Vec<_>, _>>()?;
-        let outcomes = collect_from_events(events.iter().map(|e| &e.kind));
+        let outcomes = collect_from_events(reader.events()?.map(|r| r.map(|e| e.kind)))?;
 
         Ok(Self { outcomes })
     }
 }
 
-/// Collects test outcomes from an iterator of events.
+/// Collects test outcomes from a fallible iterator of events.
 ///
 /// This helper exists to make the event processing logic testable without
-/// requiring a full `RecordReader`.
-fn collect_from_events<'a, O>(
-    events: impl Iterator<Item = &'a TestEventKindSummary<O>>,
-) -> HashMap<OwnedTestInstanceId, TestOutcome>
+/// requiring a full `RecordReader`. It accepts a fallible iterator to enable
+/// streaming without in-memory buffering.
+fn collect_from_events<K, O, E>(
+    events: impl Iterator<Item = Result<K, E>>,
+) -> Result<HashMap<OwnedTestInstanceId, TestOutcome>, E>
 where
-    O: 'a,
+    K: Borrow<TestEventKindSummary<O>>,
 {
     let mut outcomes = HashMap::new();
 
-    for kind in events {
-        match kind {
+    for kind_result in events {
+        let kind = kind_result?;
+        match kind.borrow() {
             TestEventKindSummary::Output(OutputEventKind::TestFinished {
                 test_instance,
                 run_statuses,
@@ -433,7 +470,7 @@ where
         }
     }
 
-    outcomes
+    Ok(outcomes)
 }
 
 #[cfg(test)]
@@ -454,6 +491,7 @@ mod tests {
     use proptest::prelude::*;
     use std::{
         collections::{BTreeMap, btree_map},
+        convert::Infallible,
         num::NonZero,
         sync::OnceLock,
         time::Duration,
@@ -1527,7 +1565,7 @@ mod tests {
             make_test_finished(test_regular_fail.clone(), None, false),
         ];
 
-        let outcomes = collect_from_events(events.iter());
+        let outcomes = collect_from_events(events.iter().map(Ok::<_, Infallible>)).unwrap();
 
         assert_eq!(
             outcomes.get(&test_pass_fail_pass),
@@ -1586,7 +1624,7 @@ mod tests {
             make_test_finished(test_b.clone(), Some((1, Some(2))), false),
         ];
 
-        let outcomes = collect_from_events(events.iter());
+        let outcomes = collect_from_events(events.iter().map(Ok::<_, Infallible>)).unwrap();
 
         assert_eq!(
             outcomes.get(&test_a),
