@@ -11,6 +11,6 @@ mod unit_output;
 
 pub(crate) use formatters::DisplayUnitKind;
 pub(crate) use imp::*;
-pub use progress::{MaxProgressRunning, ShowProgress};
+pub use progress::{MaxProgressRunning, ShowProgress, ShowTerminalProgress};
 pub use status_level::*;
 pub use unit_output::*;
