@@ -14,7 +14,7 @@ use crate::{
         TestRunnerBuildError, TestRunnerExecuteErrors, TracerCommandParseError,
     },
     input::{InputHandler, InputHandlerKind, InputHandlerStatus},
-    list::{TestInstanceWithSettings, TestList},
+    list::{OwnedTestInstanceId, TestInstanceWithSettings, TestList},
     reporter::events::{ReporterEvent, RunStats, StressIndex},
     runner::ExecutorEvent,
     signal::{SignalHandler, SignalHandlerKind},
@@ -22,12 +22,14 @@ use crate::{
     test_output::CaptureStrategy,
 };
 use async_scoped::TokioScope;
+use chrono::{DateTime, Local};
 use future_queue::{FutureQueueContext, StreamExt};
 use futures::{future::Fuse, prelude::*};
 use nextest_metadata::FilterMatch;
 use quick_junit::ReportUuid;
 use std::{
-    convert::Infallible, fmt, num::NonZero, pin::Pin, str::FromStr, sync::Arc, time::Duration,
+    collections::BTreeSet, convert::Infallible, fmt, num::NonZero, pin::Pin, str::FromStr,
+    sync::Arc, time::Duration,
 };
 use tokio::{
     runtime::Runtime,
@@ -225,6 +227,7 @@ pub struct TestRunnerBuilder {
     test_threads: Option<TestThreads>,
     stress_condition: Option<StressCondition>,
     interceptor: Interceptor,
+    expected_outstanding: Option<BTreeSet<OwnedTestInstanceId>>,
 }
 
 impl TestRunnerBuilder {
@@ -273,6 +276,19 @@ impl TestRunnerBuilder {
         self
     }
 
+    /// Sets the expected outstanding tests for rerun tracking.
+    ///
+    /// When set, the dispatcher will track which tests were seen during the run
+    /// and emit a `TestsNotSeen` as part of the `RunFinished` if some expected
+    /// tests were not seen.
+    pub fn set_expected_outstanding(
+        &mut self,
+        expected: BTreeSet<OwnedTestInstanceId>,
+    ) -> &mut Self {
+        self.expected_outstanding = Some(expected);
+        self
+    }
+
     /// Creates a new test runner.
     #[expect(clippy::too_many_arguments)]
     pub fn build<'a>(
@@ -308,7 +324,8 @@ impl TestRunnerBuilder {
 
         Ok(TestRunner {
             inner: TestRunnerInner {
-                run_id: ReportUuid::new_v4(),
+                run_id: force_or_new_run_id(),
+                started_at: Local::now(),
                 profile,
                 test_list,
                 test_threads,
@@ -320,6 +337,7 @@ impl TestRunnerBuilder {
                 max_fail,
                 stress_condition: self.stress_condition,
                 interceptor: self.interceptor,
+                expected_outstanding: self.expected_outstanding,
                 runtime,
             },
             signal_handler,
@@ -339,7 +357,9 @@ pub enum StressCondition {
 }
 
 /// A count for stress testing.
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+#[serde(tag = "kind", rename_all = "kebab-case")]
+#[cfg_attr(test, derive(test_strategy::Arbitrary))]
 pub enum StressCount {
     /// Run each test `count` times.
     Count {
@@ -377,6 +397,16 @@ pub struct TestRunner<'a> {
 }
 
 impl<'a> TestRunner<'a> {
+    /// Returns the unique ID for this test run.
+    pub fn run_id(&self) -> ReportUuid {
+        self.inner.run_id
+    }
+
+    /// Returns the timestamp when this test run was started.
+    pub fn started_at(&self) -> DateTime<Local> {
+        self.inner.started_at
+    }
+
     /// Returns the status of the input handler.
     pub fn input_handler_status(&self) -> InputHandlerStatus {
         self.input_handler.status()
@@ -464,6 +494,7 @@ impl<'a> TestRunner<'a> {
 #[derive(Debug)]
 struct TestRunnerInner<'a> {
     run_id: ReportUuid,
+    started_at: DateTime<Local>,
     profile: &'a EvaluatableProfile<'a>,
     test_list: &'a TestList<'a>,
     test_threads: usize,
@@ -475,6 +506,7 @@ struct TestRunnerInner<'a> {
     max_fail: MaxFail,
     stress_condition: Option<StressCondition>,
     interceptor: Interceptor,
+    expected_outstanding: Option<BTreeSet<OwnedTestInstanceId>>,
     runtime: Runtime,
 }
 
@@ -507,6 +539,7 @@ impl<'a> TestRunnerInner<'a> {
             self.max_fail,
             global_timeout,
             self.stress_condition.clone(),
+            self.expected_outstanding.clone(),
         );
 
         let executor_cx = ExecutorContext::new(
@@ -781,6 +814,25 @@ pub fn configure_handle_inheritance(
     super::os::configure_handle_inheritance_impl(no_capture)
 }
 
+/// Environment variable to force a specific run ID (for testing).
+const FORCE_RUN_ID_ENV: &str = "__NEXTEST_FORCE_RUN_ID";
+
+/// Returns a forced run ID from the environment, or generates a new one.
+fn force_or_new_run_id() -> ReportUuid {
+    if let Ok(id_str) = std::env::var(FORCE_RUN_ID_ENV) {
+        match id_str.parse::<ReportUuid>() {
+            Ok(uuid) => return uuid,
+            Err(err) => {
+                warn!(
+                    "{FORCE_RUN_ID_ENV} is set but invalid (expected UUID): {err}, \
+                     generating random ID"
+                );
+            }
+        }
+    }
+    ReportUuid::new_v4()
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
