@@ -6,9 +6,11 @@
 use super::{
     discovery::user_config_paths,
     elements::{
-        CompiledUiOverride, DefaultUiConfig, DeserializedUiConfig, DeserializedUiOverrideData,
-        UiConfig,
+        CompiledRecordOverride, CompiledUiOverride, DefaultRecordConfig, DefaultUiConfig,
+        DeserializedRecordConfig, DeserializedRecordOverrideData, DeserializedUiConfig,
+        DeserializedUiOverrideData, RecordConfig, UiConfig,
     },
+    experimental::{ExperimentalConfig, UserConfigExperimental},
 };
 use crate::errors::UserConfigError;
 use camino::Utf8Path;
@@ -17,19 +19,68 @@ use std::{collections::BTreeSet, io};
 use target_spec::{Platform, TargetSpec};
 use tracing::{debug, warn};
 
+/// Special value for `--user-config-file` and `NEXTEST_USER_CONFIG_FILE` that
+/// skips user config loading entirely.
+pub const USER_CONFIG_NONE: &str = "none";
+
+/// Specifies where to load user configuration from.
+#[derive(Clone, Copy, Debug)]
+pub enum UserConfigLocation<'a> {
+    /// Discover user config from default locations (e.g.,
+    /// `~/.config/nextest/config.toml`).
+    Default,
+
+    /// Skip user config loading entirely, using only built-in defaults.
+    ///
+    /// This is useful for test isolation.
+    Isolated,
+
+    /// Load user config from an explicit path.
+    ///
+    /// Returns an error if the file does not exist.
+    Explicit(&'a Utf8Path),
+}
+
+impl<'a> UserConfigLocation<'a> {
+    /// Creates a user config location from a CLI or environment variable value.
+    ///
+    /// Returns `Default` if `None`, `Isolated` if `"none"`, otherwise
+    /// `Explicit` with the path.
+    pub fn from_cli_or_env(s: Option<&'a str>) -> Self {
+        match s {
+            None => Self::Default,
+            Some(s) if s == USER_CONFIG_NONE => Self::Isolated,
+            Some(s) => Self::Explicit(Utf8Path::new(s)),
+        }
+    }
+}
+
 /// User configuration after custom settings and overrides have been applied.
 #[derive(Clone, Debug)]
 pub struct UserConfig {
+    /// Experimental features enabled (from config and environment variables).
+    pub experimental: BTreeSet<UserConfigExperimental>,
     /// Resolved UI configuration.
     pub ui: UiConfig,
+    /// Resolved record configuration.
+    pub record: RecordConfig,
 }
 
 impl UserConfig {
     /// Loads and resolves user configuration for the given host platform.
-    pub fn for_host_platform(host_platform: &Platform) -> Result<Self, UserConfigError> {
-        let user_config = CompiledUserConfig::from_default_location()?;
+    pub fn for_host_platform(
+        host_platform: &Platform,
+        location: UserConfigLocation<'_>,
+    ) -> Result<Self, UserConfigError> {
+        let user_config = CompiledUserConfig::from_location(location)?;
         let default_user_config = DefaultUserConfig::from_embedded();
 
+        // Combine experimental features from user config and environment variables.
+        let mut experimental = UserConfigExperimental::from_env();
+        if let Some(config) = &user_config {
+            experimental.extend(config.experimental.iter().copied());
+        }
+
         let resolved_ui = UiConfig::resolve(
             &default_user_config.ui,
             &default_user_config.ui_overrides,
@@ -41,7 +92,27 @@ impl UserConfig {
             host_platform,
         );
 
-        Ok(Self { ui: resolved_ui })
+        let resolved_record = RecordConfig::resolve(
+            &default_user_config.record,
+            &default_user_config.record_overrides,
+            user_config.as_ref().map(|c| &c.record),
+            user_config
+                .as_ref()
+                .map(|c| &c.record_overrides[..])
+                .unwrap_or(&[]),
+            host_platform,
+        );
+
+        Ok(Self {
+            experimental,
+            ui: resolved_ui,
+            record: resolved_record,
+        })
+    }
+
+    /// Returns true if the specified experimental feature is enabled.
+    pub fn is_experimental_enabled(&self, feature: UserConfigExperimental) -> bool {
+        self.experimental.contains(&feature)
     }
 }
 
@@ -91,10 +162,25 @@ impl UserConfigWarnings for DefaultUserConfigWarnings {
 #[derive(Clone, Debug, Default, Deserialize)]
 #[serde(rename_all = "kebab-case")]
 struct DeserializedUserConfig {
+    /// Experimental features to enable.
+    ///
+    /// This is a table with boolean fields for each experimental feature:
+    ///
+    /// ```toml
+    /// [experimental]
+    /// record = true
+    /// ```
+    #[serde(default)]
+    experimental: ExperimentalConfig,
+
     /// UI configuration.
     #[serde(default)]
     ui: DeserializedUiConfig,
 
+    /// Record configuration.
+    #[serde(default)]
+    record: DeserializedRecordConfig,
+
     /// Configuration overrides.
     #[serde(default)]
     overrides: Vec<DeserializedOverride>,
@@ -116,6 +202,10 @@ struct DeserializedOverride {
     /// UI settings to override.
     #[serde(default)]
     ui: DeserializedUiOverrideData,
+
+    /// Record settings to override.
+    #[serde(default)]
+    record: DeserializedRecordOverrideData,
 }
 
 impl DeserializedUserConfig {
@@ -171,6 +261,7 @@ impl DeserializedUserConfig {
     /// The `path` is used for error reporting.
     fn compile(self, path: &Utf8Path) -> Result<CompiledUserConfig, UserConfigError> {
         let mut ui_overrides = Vec::with_capacity(self.overrides.len());
+        let mut record_overrides = Vec::with_capacity(self.overrides.len());
         for (index, override_) in self.overrides.into_iter().enumerate() {
             let platform_spec = TargetSpec::new(override_.platform).map_err(|error| {
                 UserConfigError::OverridePlatformSpec {
@@ -179,12 +270,21 @@ impl DeserializedUserConfig {
                     error,
                 }
             })?;
-            ui_overrides.push(CompiledUiOverride::new(platform_spec, override_.ui));
+            // Each override entry uses the same platform spec for both UI and
+            // record settings.
+            ui_overrides.push(CompiledUiOverride::new(platform_spec.clone(), override_.ui));
+            record_overrides.push(CompiledRecordOverride::new(platform_spec, override_.record));
         }
 
+        // Convert the experimental config table to a set of enabled features.
+        let experimental = self.experimental.to_set();
+
         Ok(CompiledUserConfig {
+            experimental,
             ui: self.ui,
+            record: self.record,
             ui_overrides,
+            record_overrides,
         })
     }
 }
@@ -195,24 +295,48 @@ impl DeserializedUserConfig {
 /// expressions in overrides.
 #[derive(Clone, Debug)]
 pub(super) struct CompiledUserConfig {
+    /// Experimental features enabled in user config.
+    pub(super) experimental: BTreeSet<UserConfigExperimental>,
     /// UI configuration.
     pub(super) ui: DeserializedUiConfig,
+    /// Record configuration.
+    pub(super) record: DeserializedRecordConfig,
     /// Compiled UI overrides with parsed platform specs.
     pub(super) ui_overrides: Vec<CompiledUiOverride>,
+    /// Compiled record overrides with parsed platform specs.
+    pub(super) record_overrides: Vec<CompiledRecordOverride>,
 }
 
 impl CompiledUserConfig {
-    /// Loads and compiles user config from the default location.
-    ///
-    /// This is a convenience method that combines loading and compilation.
-    /// Platform specs in overrides are compiled and validated.
-    ///
-    /// Returns `Ok(None)` if no config file exists at any candidate path.
-    /// Returns `Err` if:
-    /// - A config file exists but cannot be read or parsed.
-    /// - A platform spec in an override is invalid.
-    pub(super) fn from_default_location() -> Result<Option<Self>, UserConfigError> {
-        Self::from_default_location_with_warnings(&mut DefaultUserConfigWarnings)
+    /// Loads and compiles user config from the specified location.
+    pub(super) fn from_location(
+        location: UserConfigLocation<'_>,
+    ) -> Result<Option<Self>, UserConfigError> {
+        Self::from_location_with_warnings(location, &mut DefaultUserConfigWarnings)
+    }
+
+    /// Loads and compiles user config from the specified location, with custom
+    /// warning handling.
+    fn from_location_with_warnings(
+        location: UserConfigLocation<'_>,
+        warnings: &mut impl UserConfigWarnings,
+    ) -> Result<Option<Self>, UserConfigError> {
+        match location {
+            UserConfigLocation::Isolated => {
+                debug!("user config: skipping (isolated)");
+                Ok(None)
+            }
+            UserConfigLocation::Explicit(path) => {
+                debug!("user config: loading from explicit path {path}");
+                match Self::from_path_with_warnings(path, warnings)? {
+                    Some(config) => Ok(Some(config)),
+                    None => Err(UserConfigError::FileNotFound {
+                        path: path.to_owned(),
+                    }),
+                }
+            }
+            UserConfigLocation::Default => Self::from_default_location_with_warnings(warnings),
+        }
     }
 
     /// Loads and compiles user config from the default location, with custom
@@ -263,6 +387,9 @@ struct DeserializedDefaultUserConfig {
     /// UI configuration (base settings, all required).
     ui: DefaultUiConfig,
 
+    /// Record configuration (base settings, all required).
+    record: DefaultRecordConfig,
+
     /// Configuration overrides.
     #[serde(default)]
     overrides: Vec<DeserializedOverride>,
@@ -277,8 +404,14 @@ pub(super) struct DefaultUserConfig {
     /// Base UI configuration.
     pub(super) ui: DefaultUiConfig,
 
+    /// Base record configuration.
+    pub(super) record: DefaultRecordConfig,
+
     /// Compiled UI overrides with parsed platform specs.
     pub(super) ui_overrides: Vec<CompiledUiOverride>,
+
+    /// Compiled record overrides with parsed platform specs.
+    pub(super) record_overrides: Vec<CompiledRecordOverride>,
 }
 
 impl DefaultUserConfig {
@@ -309,24 +442,26 @@ impl DefaultUserConfig {
         }
 
         // Compile platform specs in overrides.
-        let ui_overrides: Vec<CompiledUiOverride> = config
-            .overrides
-            .into_iter()
-            .enumerate()
-            .map(|(index, override_)| {
-                let platform_spec = TargetSpec::new(override_.platform).unwrap_or_else(|error| {
-                    panic!(
-                        "embedded default user config has invalid platform spec \
-                         in [[overrides]] at index {index}: {error}"
-                    )
-                });
-                CompiledUiOverride::new(platform_spec, override_.ui)
-            })
-            .collect();
+        let mut ui_overrides = Vec::with_capacity(config.overrides.len());
+        let mut record_overrides = Vec::with_capacity(config.overrides.len());
+        for (index, override_) in config.overrides.into_iter().enumerate() {
+            let platform_spec = TargetSpec::new(override_.platform).unwrap_or_else(|error| {
+                panic!(
+                    "embedded default user config has invalid platform spec \
+                     in [[overrides]] at index {index}: {error}"
+                )
+            });
+            // Each override entry uses the same platform spec for both UI and
+            // record settings.
+            ui_overrides.push(CompiledUiOverride::new(platform_spec.clone(), override_.ui));
+            record_overrides.push(CompiledRecordOverride::new(platform_spec, override_.record));
+        }
 
         Self {
             ui: config.ui,
+            record: config.record,
             ui_overrides,
+            record_overrides,
         }
     }
 }
@@ -450,7 +585,75 @@ mod tests {
             warnings.unknown_keys.is_none(),
             "no unknown keys should be detected"
         );
-        assert_eq!(config.ui_overrides.len(), 2, "should have 2 overrides");
+        assert_eq!(config.ui_overrides.len(), 2, "should have 2 UI overrides");
+        assert_eq!(
+            config.record_overrides.len(),
+            2,
+            "should have 2 record overrides"
+        );
+    }
+
+    #[test]
+    fn overrides_record_parsing() {
+        let config_contents = r#"
+        [record]
+        enabled = false
+
+        [[overrides]]
+        platform = "cfg(unix)"
+        record.enabled = true
+        record.max-output-size = "50MB"
+
+        [[overrides]]
+        platform = "cfg(windows)"
+        record.enabled = true
+        record.max-records = 200
+        "#;
+
+        let temp_dir = tempdir().unwrap();
+        let config_path = temp_dir.path().join("config.toml");
+        std::fs::write(&config_path, config_contents).unwrap();
+
+        let mut warnings = TestUserConfigWarnings::default();
+        let config = CompiledUserConfig::from_path_with_warnings(&config_path, &mut warnings)
+            .expect("config valid")
+            .expect("config should exist");
+
+        assert!(
+            warnings.unknown_keys.is_none(),
+            "no unknown keys should be detected"
+        );
+        assert_eq!(
+            config.record_overrides.len(),
+            2,
+            "should have 2 record overrides"
+        );
+    }
+
+    #[test]
+    fn overrides_record_unknown_key() {
+        let config_contents = r#"
+        [[overrides]]
+        platform = "cfg(unix)"
+        record.enabled = true
+        record.unknown-key = "test"
+        "#;
+
+        let temp_dir = tempdir().unwrap();
+        let config_path = temp_dir.path().join("config.toml");
+        std::fs::write(&config_path, config_contents).unwrap();
+
+        let mut warnings = TestUserConfigWarnings::default();
+        let _config = CompiledUserConfig::from_path_with_warnings(&config_path, &mut warnings)
+            .expect("config valid")
+            .expect("config should exist");
+
+        let (path, unknown) = warnings.unknown_keys.expect("should have unknown keys");
+        assert_eq!(path, config_path, "path should match");
+        assert!(
+            unknown.contains("overrides.0.record.unknown-key"),
+            "unknown key should be detected: {unknown:?}"
+        );
     }
 
     #[test]
@@ -503,4 +706,103 @@ mod tests {
             "should fail with parse error due to missing required platform field: {result:?}"
         );
     }
+
+    #[test]
+    fn experimental_features_parsing() {
+        let config_contents = r#"
+        [experimental]
+        record = true
+
+        [ui]
+        show-progress = "bar"
+        "#;
+
+        let temp_dir = tempdir().unwrap();
+        let config_path = temp_dir.path().join("config.toml");
+        std::fs::write(&config_path, config_contents).unwrap();
+
+        let mut warnings = TestUserConfigWarnings::default();
+        let config = CompiledUserConfig::from_path_with_warnings(&config_path, &mut warnings)
+            .expect("config valid")
+            .expect("config should exist");
+
+        assert!(
+            warnings.unknown_keys.is_none(),
+            "no unknown keys should be detected"
+        );
+        assert!(
+            config
+                .experimental
+                .contains(&UserConfigExperimental::Record),
+            "record feature should be enabled"
+        );
+    }
+
+    #[test]
+    fn experimental_features_disabled() {
+        let config_contents = r#"
+        [experimental]
+        record = false
+
+        [ui]
+        show-progress = "bar"
+        "#;
+
+        let temp_dir = tempdir().unwrap();
+        let config_path = temp_dir.path().join("config.toml");
+        std::fs::write(&config_path, config_contents).unwrap();
+
+        let mut warnings = TestUserConfigWarnings::default();
+        let config = CompiledUserConfig::from_path_with_warnings(&config_path, &mut warnings)
+            .expect("config valid")
+            .expect("config should exist");
+
+        assert!(
+            warnings.unknown_keys.is_none(),
+            "no unknown keys should be detected"
+        );
+        assert!(
+            !config
+                .experimental
+                .contains(&UserConfigExperimental::Record),
+            "record feature should not be enabled"
+        );
+    }
+
+    #[test]
+    fn experimental_features_unknown_warning() {
+        let config_contents = r#"
+        [experimental]
+        record = true
+        unknown-feature = true
+
+        [ui]
+        show-progress = "bar"
+        "#;
+
+        let temp_dir = tempdir().unwrap();
+        let config_path = temp_dir.path().join("config.toml");
+        std::fs::write(&config_path, config_contents).unwrap();
+
+        let mut warnings = TestUserConfigWarnings::default();
+        let config = CompiledUserConfig::from_path_with_warnings(&config_path, &mut warnings)
+            .expect("config valid")
+            .expect("config should exist");
+
+        // Unknown fields should be warnings, not errors.
+        let (path, unknown) = warnings.unknown_keys.expect("should have unknown keys");
+        assert_eq!(path, config_path, "path should match");
+        assert!(
+            unknown.contains("experimental.unknown-feature"),
+            "unknown key should be detected: {unknown:?}"
+        );
+
+        // The known feature should still be enabled.
+        assert!(
+            config
+                .experimental
+                .contains(&UserConfigExperimental::Record),
+            "record feature should be enabled"
+        );
+    }
 }
